@@ -9,23 +9,20 @@
 -------------------------------------------------
 """
 import inspect
-import logging
 import os
+import random
 import time
-from concurrent.futures import ThreadPoolExecutor
+from concurrent.futures import ThreadPoolExecutor, ProcessPoolExecutor
 from functools import wraps
-from typing import Generator, Iterable, List, Tuple, Union
+from typing import Generator, Iterable, List, Tuple
 
+
+from loguru import logger as default_logger
 from tqdm import tqdm
 
-from snippets.utils import jload
-
-default_logger = logging.getLogger(__file__)
-default_logger.setLevel(logging.DEBUG)
-
 
 # 输出function执行耗时的函数
-def log_cost_time(name=None, level=logging.INFO, logger=None, star_len=0):
+def log_cost_time(name=None, level="INFO", logger=None, star_len=0):
     def wrapper(func):
         @wraps(func)
         def wrapped(*args, **kwargs):
@@ -38,9 +35,9 @@ def log_cost_time(name=None, level=logging.INFO, logger=None, star_len=0):
 
     return wrapper
 
-
+# 输出执行时长的包装器
 class LogCostContext(object):
-    def __init__(self, name, level=logging.INFO, logger=None, star_len=0):
+    def __init__(self, name, level="INFO", logger=None, star_len=0):
         self.name = name
         self.level = level
         self.star_len = star_len
@@ -50,7 +47,7 @@ class LogCostContext(object):
         msg = f"{self.name} starts"
         half_star_len = max((self.star_len - len(msg)) // 2, 0)
         msg = "*" * half_star_len + msg + "*" * half_star_len
-        self.logger.log(msg=msg, level=self.level)
+        self.logger.log(self.level, msg)
         self.st = time.time()
 
     def __exit__(self, type, value, traceback):
@@ -58,12 +55,13 @@ class LogCostContext(object):
         msg = f"{self.name} ends, cost:{cost:4.3f} seconds"
         half_star_len = max((self.star_len - len(msg)) // 2, 0)
         msg = "*" * half_star_len + msg + "*" * half_star_len
-        self.logger.log(msg=msg, level=self.level)
+        self.logger.log(self.level, msg)
 
 
 # 执行函数时输出函数的参数以及返回值
-def log_function_info(input_level=logging.DEBUG, result_level=logging.DEBUG,
+def log_function_info(input_level="DEBUG", result_level="DEBUG",
                       exclude_self=True):
+
     def wrapper(func):
         def wrapped_func(*args, **kwargs):
             if input_level:
@@ -71,12 +69,12 @@ def log_function_info(input_level=logging.DEBUG, result_level=logging.DEBUG,
                 if exclude_self and len(args) > 1:
                     show_args = args[1:]
                 msg = f"call function:{func} with\n args:{show_args}\n kwargs:{kwargs}"
-                default_logger.log(level=input_level, msg=msg)
+                default_logger.log(input_level, msg)
 
             res = func(*args, **kwargs)
             if result_level:
                 msg = f"function:{func} return with:\n{res}"
-                default_logger.log(level=result_level, msg=msg)
+                default_logger.log(result_level, msg)
             return res
 
         return wrapped_func
@@ -90,9 +88,7 @@ def ensure_file_path(func):
     def wrapper(*args, **kwargs):
         path = kwargs['path']
         dir_path = os.path.dirname(path)
-        if not os.path.exists(dir_path):
-            os.makedirs(dir_path)
-        return func(*args, **kwargs)
+        os.makedirs(dir_path, exist_ok=True)
 
     return wrapper
 
@@ -102,8 +98,7 @@ def ensure_dir_path(func):
     @wraps(func)
     def wrapper(*args, **kwargs):
         dir_path = kwargs['path']
-        if not os.path.exists(dir_path):
-            os.makedirs(dir_path)
+        os.makedirs(dir_path, exist_ok=True)
         return func(*args, **kwargs)
 
     return wrapper
@@ -112,7 +107,6 @@ def ensure_dir_path(func):
 # 忽略过多的kwarg参数
 def discard_kwarg(func):
     arg_names = inspect.signature(func).parameters.keys()
-
     def wrap(*args, **kwargs):
         kwargs = {k: v for k, v in kwargs.items() if k in arg_names}
         return func(*args, **kwargs)
@@ -120,7 +114,7 @@ def discard_kwarg(func):
 
 
 # adapt function with single elements
-def adapt_single(ele_name):
+def adapt_single(ele_name:str):
     def wrapper(func):
         @wraps(func)
         def wrapped(*args, **kwargs):
@@ -138,8 +132,8 @@ def adapt_single(ele_name):
         return wrapped
     return wrapper
 
-
-def retry(retry_num, wait_time, level=logging.INFO):
+# 自动加上重试功能
+def retry(retry_num: int, wait_time: float | tuple[float, float], level="INFO"):
     def wrapper(func):
         @wraps(func)
         def wrapped(*args, **kwargs):
@@ -150,42 +144,66 @@ def retry(retry_num, wait_time, level=logging.INFO):
                     return func(*args, **kwargs)
                 except Exception as e:
                     if num_left == 0:
+                        default_logger.warning("no attempts left, throw exception")
+                        default_logger.exception(e)
                         raise e
-                    time.sleep(wait_time)
-                    default_logger.log(
-                        level, f'retry {func.__name__}, {num_left} retry left')
+                    if isinstance(wait_time, tuple):
+                        wt = random.uniform(wait_time[0], wait_time[1])
+                    else:
+                        wt = wait_time
+
+                    default_logger.log(level, f'retry {func.__name__}, {num_left} attempts left, sleep:{wt:2.3f} seconds')
+                    time.sleep(wt)
                     num_left -= 1
         return wrapped
     return wrapper
 
 
 # 线程池批量跑function
-def batch_process(work_num, return_list=False):
+def multi_thread(work_num, return_list=False):
     def wrapper(func):
         @wraps(func)
         def wrapped(data: Iterable, *args, **kwargs):
-            # add a thread pool here
-            executors = ThreadPoolExecutor(work_num)
-
-            def _func(x):                
+            def _func(x):
                 return func(x, *args, **kwargs)
+            
+            executors = ThreadPoolExecutor(work_num)
             rs_iter = executors.map(_func, data)
             total = None if not hasattr(data, '__len__') else len(data)
-            rs = tqdm(rs_iter, total=total)
-            if return_list:
-                return list(rs)
-            return rs
+            rs_iter = tqdm(rs_iter, total=total)
+            return list(rs_iter) if return_list else rs_iter
+        return wrapped
+    return wrapper
+
+batch_process = multi_thread
+
+# 多进程不可以使用内部定义的function
+def multi_process(work_num,  return_list=False):
+    def wrapper(func):
+        @wraps(func)
+        def wrapped(data: Iterable):
+            executors = ProcessPoolExecutor(work_num)
+            # with ProcessPoolExecutor(work_num) as executors:
+            rs_iter = executors.map(func, data)
+            rs = rs_iter
+            total = None if not hasattr(data, '__len__') else len(data)
+            rs_iter = tqdm(rs_iter, total=total)
+            return list(rs_iter) if return_list else rs
+
         return wrapped
     return wrapper
 
 
-class ConfigMixin:
-    @classmethod
-    def from_config(cls, config: Union[dict, str]):
-        if isinstance(config, str):
-            if config.endswith(".json"):
-                config = jload(config)
-            else:
-                raise ValueError(f"{config} is not a valid config file")
-        instance = cls(**config)
-        return instance
+
+
+if __name__ == "__main__":
+    work_num = 4
+    executors = ProcessPoolExecutor(work_num)
+    def add1(a):
+        print(f"process {a}")
+        return a + 1
+    with executors:
+        rs_iter = executors.map(add1, range(10))
+        for e in rs_iter:
+            print(e)
+
