@@ -18,7 +18,10 @@ class LoguruFormat(str, Enum):
     FILE_DETAIL = "{time:YYYY-MM-DD HH:mm:ss.SSS} [{level: <8}] | {name}:{line}[{function}] - {message}"
 
 
-def set_logger(env: str, module_name:str, log_dir=None, log_path=None):
+handlers = dict()
+
+
+def set_logger(env: str, module_name: str, log_dir=None, log_path=None):
     """_summary_
 
     Args:
@@ -35,10 +38,10 @@ def set_logger(env: str, module_name:str, log_dir=None, log_path=None):
     fmt = LoguruFormat.DETAIL if env in ["dev", "local"] else LoguruFormat.SIMPLE
     level = "DEBUG" if env in ["dev", "local"] else "INFO"
     retention = "7 days" if env in ["dev", "local"] else "30 days"
-    filter= lambda r: module_name in r["name"]
-    logger.add(sys.stdout, colorize=True, format=fmt, level=level,
-            #    serialize = True,
-               filter=filter)
+    def filter(r): return module_name in r["name"]
+    handler_id = logger.add(sys.stdout, colorize=True, format=fmt, level=level, filter=filter)
+    handlers[f"{module_name}_stdout"] = handler_id
+
     if log_path:
         logger.add(log_path, rotation="00:00", retention=retention, enqueue=True, backtrace=True, level=level, filter=filter)
     if log_dir:
@@ -48,3 +51,41 @@ def set_logger(env: str, module_name:str, log_dir=None, log_path=None):
         output_log_path = os.path.join(log_dir, "output.log")
         logger.add(output_log_path,  rotation="00:00", retention=retention, enqueue=True, backtrace=True, level="INFO", filter=filter)
     return logger
+
+
+def get_handler(module_name, sink_type):
+    key = "_".join([module_name, sink_type])
+    if key not in handlers:
+        return None
+    handler_id = handlers[key]
+    handler = logger._core.handlers[handler_id]
+    return handler
+
+
+def update_level(module_name, sink_type, level):
+    handler = get_handler(module_name, sink_type)
+    level_no = logger.level(level).no
+    # print(f"{level_no=}")
+    handler._levelno = level_no
+
+
+# 输出执行时长的包装器
+class ChangeLogLevelContext(object):
+    def __init__(self, module_name, sink_type, level):
+        self.module_name = module_name
+        self.level = level
+        self.sink_type = sink_type
+        self.handler = get_handler(module_name, sink_type)
+        self.old_level = None
+
+    def __enter__(self):
+        if self.handler and self.level:
+            self.old_level = self.handler._levelno
+            level_no = logger.level(self.level).no
+            # print(f"set log level :{level_no}")
+            self.handler._levelno = level_no
+
+    def __exit__(self, type, value, traceback):
+        if self.handler and self.old_level:
+            # print(f"restore log level :{self.old_level}")
+            self.handler._levelno = self.old_level
